// SVEHaTypCross strategy is Copyright (C) 2013, Sylvain Vervoort <stocata.org>.
// stocata.org reserves the right to modify/overwrite this NinjaScript with each release.
// Release V1.1 February, 2013.
// This strategy is for training purposes only. Any other use is strictly at your own risk.

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
using System.IO;
using System.Collections;
using System.Windows.Forms;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Typical price cross over with Heikin-Ashi prices for stocks
    /// </summary>
    [Description("SVEHaTypCross Price cross over system for trading Stocks")]
    public class SVEHaTypCross : Strategy
  {
	// Variables----------------------------------------------------------------------------
	#region Variables
	    private int 	haCaverage 		= 8;	// Heikin-Ashi average
        private int 	typicalaverage	= 5;	// Typical price average
		private double 	stoplossperc 	= 25;	// Stop loss percentage
		private double 	breakevenperc 	= 15;	// Break even percentage
		private int     stocksqty 		= 100;	// Basic stock buying quantity
		
		bool OpnBuyClsSell = false, OpnSellClsBuy = false;
		
		private IOrder sOrderLong 		= null; 	// Long order identity
		private IOrder sStopLong 		= null;		// Stop loss long order identity
		private IOrder sOrderShort 		= null; 	// Short order identity
		private IOrder sStopShort 		= null;		// Stop loss short order identity
		private IOrder sOrdExLong 		= null; 	// Exit long order identity
		private IOrder sOrdExShort 		= null;		// Exit short order identity
	
		#endregion

    // Initialize ----------------------------------------------------------------------------
    protected override void Initialize()
        {	
		CalculateOnBarClose = true;
		TraceOrders 		= true;
		EntryHandling 		= EntryHandling.UniqueEntries;
		ExitOnClose 		= false; 
		SyncAccountPosition = false;
		StopTargetHandling 	= StopTargetHandling.ByStrategyPosition;
		}
	
	// On bar update  ----------------------------------------------------------------------------
    protected override void OnBarUpdate()
    {	
		OpnBuyClsSell = false; OpnSellClsBuy = false;
					
			// The single autotrade signal crossovers
			if (SVEHaTypCrossInd(haCaverage,typicalaverage)[0] > 0 &&
				SVEHaTypCrossInd(haCaverage,typicalaverage)[1] < 1)
				OpnBuyClsSell = true;

			if (SVEHaTypCrossInd(haCaverage,typicalaverage)[0] < 1 &&
				SVEHaTypCrossInd(haCaverage,typicalaverage)[1] > 0)
				OpnSellClsBuy = true;
			
			// Opening the trade
			if (OpnBuyClsSell == true && Position.MarketPosition == MarketPosition.Flat) 
				sOrderLong = EnterLong(stocksqty,"sOrderLong");
			else if (OpnBuyClsSell == true && Position.MarketPosition == MarketPosition.Short)
			{
				sOrdExShort = ExitShort(Position.Quantity,"sOrdExShort","sOrderShort");
				sOrderLong 	= EnterLong(stocksqty,"sOrderLong");
			}
			
			if (OpnSellClsBuy == true && Position.MarketPosition == MarketPosition.Flat) 		
				sOrderShort = EnterShort(stocksqty, "sOrderShort");
			else if (OpnSellClsBuy == true && Position.MarketPosition == MarketPosition.Long)
			{
				sOrdExLong 	= ExitLong(Position.Quantity,"sOrdExLong","sOrderLong");
				sOrderShort = EnterShort(stocksqty,"sOrderShort");
			}
			
			// Modify stop-loss setting to breakeven for a long position
			if (Position.MarketPosition == MarketPosition.Long && 
				Close[0] >= Position.AvgPrice + breakevenperc * 0.01 * Position.AvgPrice)
			{
				// Check if a Stop Order exists, modify it to breakeven
				if (sStopLong != null && sStopLong.StopPrice < Position.AvgPrice)
				{
				// Modify stop-loss to breakeven 
				sStopLong = ExitLongStop(0,true, sStopLong.Quantity, 
							Position.AvgPrice, "sStopLong", "sOrderLong");
				}
			}
			// Modify stop-loss to breakeven for a short position
			if (Position.MarketPosition == MarketPosition.Short && 
				Close[0] <= Position.AvgPrice - breakevenperc * 0.01 * Position.AvgPrice)
			{
				// Check if a Stop Order exists, modify it to breakeven
				if (sStopShort != null && sStopShort.StopPrice > Position.AvgPrice)
				{
				// Modify stop-loss to breakeven 
				sStopShort = ExitShortStop(0,true, sStopShort.Quantity, 
							 Position.AvgPrice, "sStopShort", "sOrderShort");
				}
			}
    }
		
		// -----------------------------------------------------------------------------------------
        protected override void OnOrderUpdate(IOrder order)
        {
			// Identify the entry order by calling the OnOrderUpdate() method.
			if (sOrderLong != null && sOrderLong == order)
				{	
				// Reset the entryOrder object to null if order was cancelled without any fill
				if (order.OrderState == OrderState.Cancelled && order.Filled == 0)
					{
					sOrderLong = null;
					}
				}
			if (sOrderShort != null && sOrderShort == order)
				{	
				if (order.OrderState == OrderState.Cancelled && order.Filled == 0)
					{
					sOrderShort = null;
					}
				}
		}
		
		// ------------------------------------------------------------------------------------------
        protected override void OnExecution(IExecution execution)
        {
			// Monitoring order Execution to submit a stop order for a long position.
			// OnExecution() is called after OnOrderUpdate().
			if (sOrderLong != null && sOrderLong == execution.Order)
			{
				if (execution.Order.OrderState == OrderState.Filled || 
					execution.Order.OrderState == OrderState.PartFilled || 
				   (execution.Order.OrderState == OrderState.Cancelled && execution.Order.Filled > 0))
				{
					sStopLong = ExitLongStop(0, true, execution.Order.Filled, 
					  execution.Order.AvgFillPrice - stoplossperc * 0.01 * execution.Order.AvgFillPrice, 
						"sStopLong", "sOrderLong");
					
					// Resetting the entryOrder object to null after the order is filled
					if (execution.Order.OrderState != OrderState.PartFilled) 	
					sOrderLong = null;
				}
			}
			
			// Reset exit long order when position is closed
			if (sOrdExLong == execution.Order) 
			{
				sOrdExLong	 	= null; 
			}
			
			// Reset when stop long position is closed
			if (sStopLong == execution.Order) 
			{
				sStopLong	= null;
			}
			
			// Submitting stop order for a short position...
			if (sOrderShort != null && sOrderShort == execution.Order)
			{
				if (execution.Order.OrderState == OrderState.Filled || 
					execution.Order.OrderState == OrderState.PartFilled || 
				   (execution.Order.OrderState == OrderState.Cancelled && execution.Order.Filled > 0))
				{
					sStopShort = ExitShortStop(0, true, execution.Order.Filled, 
					  execution.Order.AvgFillPrice + stoplossperc * 0.01 * execution.Order.AvgFillPrice, 
						"sStopShort", "sOrderShort");
					
					// Resetting the entryOrder object to null after the order is filled
					if (execution.Order.OrderState != OrderState.PartFilled) 
					sOrderShort = null;
				}
			}
			
			// Reset exit short order when position is closed
			if (sOrdExShort == execution.Order) 
			{
				sOrdExShort	 	= null; 
			}
			
			// Reset when stop short position is closed
			if (sStopShort == execution.Order) 
			{
				sStopShort	= null;
			}
						

		}

        #region Properties
		
		[Description("Heikin-Ashi average: DEFAULT = 8")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("1. Heikin-Ashi average")]
        public int P01_haCaverage
        {
            get { return haCaverage; }
            set { haCaverage = Math.Max(1, value); }
        }
		
        [Description("Typical average: DEFAULT = 5")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("2. Typical average")]
        public int P02_typicalaverage
        {
            get { return typicalaverage; }
            set { typicalaverage = Math.Max(1, value); }
        }

		[Description("Emergency stop percentage: DEFAULT = 25.0 percent")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("3. Emergency stop percentage")]
        public double P05_stoplossPercentage
        {
            get { return stoplossperc; }
            set { stoplossperc = Math.Max(0.001, value); }
        }
		
		[Description("Breakeven percentage: DEFAULT = 15.0 percent")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("4. Breakeven percentage")]
        public double P06_breakevenPercentage
        {
            get { return breakevenperc; }
            set { breakevenperc = Math.Max(0.001, value); }
        }		
				
		[Description("Number of stocks traded: DEFAULT = 100")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("5. Buy/Sell quantity")]
        public int P07_stocks_Quantity
        {
            get { return stocksqty; }
            set { stocksqty = Math.Max(1, value); }
        }			
		
        #endregion
	}
}
